#include "CommunicationManager.h"
#include "Communication.h"
#include "ConnectionSettings.h"
#include "Meeting.h"
#include "Room.h"
#include <QDateTime>
#include <QDomDocument>
#include <QDebug>

static const int ERROR_BASE=100;

CommunicationManager::CommunicationManager( const ConnectionSettings &aConnection )
{
	iConnectionSettings = new ConnectionSettings( aConnection );
	iModifyingCommunication = NULL;
	iFetchingCommunication = new Communication( aConnection );

	if ( iFetchingCommunication )
	{
		connect( iFetchingCommunication,
				 SIGNAL( readProgress( int, int, int ) ),
				 this,
				 SLOT( readProgress( int, int, int ) )
				);
		connect( iFetchingCommunication,
				 SIGNAL( requestFinished( int, int ) ),
				 this,
				 SLOT( requestFinished( int, int ) )
				);
	}
}

CommunicationManager::~CommunicationManager()
{
	delete iConnectionSettings;
	delete iModifyingCommunication;
	delete iFetchingCommunication;
	while ( !iMeetings.isEmpty() )
		delete iMeetings.takeFirst();
}

void CommunicationManager::fetchMeetings( const int aWeek, const int aYear, const Room &aIn )
{
	//prevent making multiple simultaneous user availibility requests
	//TODO: Would be nice to queue these requests instead of just aborting 
	const RequestData* rd = findRequest( GetUserAvailability );
	if( rd )
		return;
	
	// Set from to beginning, 00:00 of the requested week
	QDateTime from = QDateTime::currentDateTime();
	from = from.addYears( aYear - from.date().year() );
	from = from.addDays( -1*(from.date().dayOfWeek()-1) );
	from = from.addDays( (aWeek - from.date().weekNumber())*7 );
	QTime midnight = from.time();
	midnight.setHMS( 0,0,0 );
	from.setTime( midnight );
	qDebug() << "CommunicationManager::fetchMeetings from: " << from.toString();
	
	QDateTime until = from.addDays(7);
	qDebug() << "CommunicationManager::fetchMeetings until: " << until.toString();
	
	
	ReqMsgGetUserAvailability msg;
	msg.setTimeZone();
	msg.setTimeWindow( from, until );
	msg.addUser( aIn.address() );
		
	int id = iFetchingCommunication->request( msg.getContentTypeForHeader(), msg.getMessage() );
	qDebug() << "CommunicationManager::fetchMeetings: id: " << id;
	if( id )
	{
		addRequest( GetUserAvailability, id );
		iRequestInfos.first()->room = new Room( aIn );
	}
}

void CommunicationManager::getSecondaryIdForMeeting( Meeting &aMeeting )
{
	ReqMsgConvertMeetingId msg( aMeeting.primaryId(), aMeeting.room().address() );
	
	QByteArray arr = msg.getMessage();

	int id = iFetchingCommunication->request( msg.getContentTypeForHeader(), arr );
	qDebug() << "CommunicationManager::getSecondaryIdForMeeting: id: " << id;
	if( id )
	{
		addRequest( ConvertId, id );
		iRequestInfos.first()->meeting = &aMeeting;
	}
}

void CommunicationManager::fetchMeetingDetails( Meeting& aMeeting )
{
	if( aMeeting.detailsAvailable() )
	{
		emit meetingDetailsFetched( aMeeting );
		return;
	}
	if( aMeeting.secondaryId().isEmpty() )
	{
		getSecondaryIdForMeeting( aMeeting );
	}
	else
	{
		getMeetingDetails( aMeeting );
	}
}

void CommunicationManager::cancelFetchMeetingDetails()
{
	const RequestData *rd = findRequest( GetCalendarItem );
	if( rd != NULL ) {
		int id = rd->requestId;
		iCancelledRequests.append( id );
	}
}

void CommunicationManager::getMeetingDetails( Meeting &aMeeting )
{
	ReqMsgGetCalendarItem msg( aMeeting.secondaryId() );
	
	QByteArray arr = msg.getMessage();
	
	int id = iFetchingCommunication->request( msg.getContentTypeForHeader(), arr );
	qDebug() << "CommunicationManager::getMeetingDetails: id: " << id;
	if( id )
	{
		addRequest( GetCalendarItem, id );
		iRequestInfos.first()->meeting = &aMeeting;
	}
}

void CommunicationManager::requestFinished( int aRequestId, int aError )
{
	RequestData* rd = takeRequest( aRequestId );
	if( iCancelledRequests.contains( rd->requestId ) )
	{
		iCancelledRequests.removeAll( rd->requestId );
		delete rd;
		return;
	}
	QByteArray* response = iFetchingCommunication->response( aRequestId );
	qDebug() << "CommunicationManager::requestFinished: id: " << aRequestId << " error: " << aError;

	if( aError != (int)(QHttp::NoError) || rd == NULL || response == NULL )
	{
		int err = aError;
		if( rd == NULL || response == NULL )
			err = 10;
		delete rd;
		emit error( ERROR_BASE+(int)err, CommunicationManager::FetchingCommunication );
		return;
	}

	switch( rd->type )
	{
	case GetUserAvailability:
		{
		ResMsgGetUserAvailability msg( *response );
	
		while ( !iMeetings.isEmpty() )
			delete iMeetings.takeFirst();
	
		int err = msg.getMeetingsFromResponse( iMeetings, *(rd->room) );
		if( err )
			emit error( ERROR_BASE+8, CommunicationManager::FetchingCommunication );
		else
			emit meetingsFetched( iMeetings );
		break;
		}
	case ConvertId:
		{
		ResponseMessage msg( *response );
		QString id = msg.getNodeValue( QString( "Id" ),
									   QDomNode::AttributeNode,
									   QString( "AlternateId" ) );
		qDebug( "ID IS: %s", id.toStdString().data() );
		rd->meeting->setSecondaryId( id );
		getMeetingDetails( *(rd->meeting) );
		break;
		}
	case GetCalendarItem:
		{
		ResMsgGetCalendarItem msg( *response );
		int err = msg.getMeetingDetailsFromResponse( *(rd->meeting) );
		if( err )
			emit error( ERROR_BASE+9, CommunicationManager::FetchingCommunication );
		else
			emit meetingDetailsFetched( *(rd->meeting) );
		break;
		}
	}
	delete response;
	delete rd;
}

void CommunicationManager::readProgress( int /*aRequestId*/, int aDone, int aTotal )
{
	emit readProgress( aDone, aTotal, CommunicationManager::FetchingCommunication );
}

void CommunicationManager::addRequest( RequestType aType, int aRequestId )
{
	RequestData* rd = new RequestData( aType, aRequestId );
	iRequestInfos.append( rd );
}

CommunicationManager::RequestData* CommunicationManager::takeRequest( int aRequestId )
{
	if( aRequestId == 0 )
		return NULL;

	for( int i = iRequestInfos.count() - 1; i >= 0 ; i-- )
	{
		struct RequestData* rd = iRequestInfos[i];
		if( rd->requestId == aRequestId )
		{
			iRequestInfos.removeAt( i );
			return rd;
		}
	}
	return NULL;
}

const CommunicationManager::RequestData* CommunicationManager::findRequest( RequestType aRequestType ) const
{
	for( int i = iRequestInfos.count() - 1; i >= 0 ; i-- )
	{
		struct RequestData* rd = iRequestInfos[i];
		if( rd->type == aRequestType )
		{
			return rd;
		}
	}
	return NULL;
}
