#include "DeviceManager.h"
#include "AlarmSender.h"
#include "HWKeyListener.h"
#include "StartupSettings.h"
#include "DeviceDataStorage.h"
#include "DeviceConfigurator.h"
#include "OperationModeToggler.h"

#include <QtDebug>
#include <QStringList>

DeviceManager::DeviceManager( StartupSettings *aSettings ) :
	iAlarmSender( 0 ),
	iHWKeyListener( 0 ),
	iSettings( 0 ),
	iDataStorage( 0 ),
	iConfigurator( 0 ),
	iModeToggler( 0 )
{
	qDebug() << "DeviceManager::DeviceManager( StartupSettings * )";

	iSettings = aSettings;
	iSendErrorMessages = true;
}

DeviceManager::~DeviceManager()
{
	qDebug() << "DeviceManager::~DeviceManager()";
	delete iDataStorage;
	iDataStorage = 0;
	delete iAlarmSender;
	iAlarmSender = 0;
	delete iConfigurator;
	iConfigurator = 0;
	delete iDataStorage;
	iDataStorage = 0;
	delete iHWKeyListener;
	iHWKeyListener = 0;
}

void DeviceManager::initDeviceManager()
{
	qDebug() << "DeviceManager::init()";
	
	qRegisterMetaType<DeviceManager::ErrorCode>("DeviceManager::ErrorCode");
	
	iDataStorage = new DeviceDataStorage();
	connect( iDataStorage, SIGNAL( dataStorageInitFailed( DeviceManager::ErrorCode ) ),
			this, SLOT( errorSender( DeviceManager::ErrorCode ) ) );

	iAlarmSender = new AlarmSender( iDataStorage );
	connect( iAlarmSender, SIGNAL( alarmSendingFailed( DeviceManager::ErrorCode, const QString& ) ),
			this, SLOT( errorSender( DeviceManager::ErrorCode, const QString& ) ) );

	iConfigurator = new DeviceConfigurator( iDataStorage );
	connect( iConfigurator, SIGNAL( configuringError( DeviceManager::ErrorCode ) ),
			this, SLOT( errorSender( DeviceManager::ErrorCode ) ) );

	if ( !iDataStorage->initDataStorage() || !setCurrentOperationMode() )
		iMode = EmptyMode;

	iHWKeyListener = new HWKeyListener();
	handleHWKeyPresses( true );
}

DeviceManager::OperationMode DeviceManager::currentOperationMode()
{
	return iMode;
}

QString DeviceManager::operationModeToString( OperationMode aMode )
{
	switch ( aMode )
	{
		case KioskMode:
			return "Kiosk-mode";
		case StandAloneMode:
			return "Stand Alone-mode";
		default:
			return "";
	}
}

void DeviceManager::changeMode()
{
	qDebug() << "void DeviceManager::changeMode()";
	if( iModeToggler == 0 ) {
		iModeToggler = new OperationModeToggler( iMode, iSettings, iAlarmSender, iConfigurator, iDataStorage );
		connect( iModeToggler, SIGNAL( finished() ), this, SLOT( modeChanged() ) );
		connect( iModeToggler, SIGNAL( error( DeviceManager::ErrorCode, const QString & ) ),
				this, SLOT( errorSender( DeviceManager::ErrorCode, const QString & ) ) );
		connect( iModeToggler, SIGNAL( changingMode( const QString & ) ), this, SIGNAL( changingMode( const QString & ) ) );
		connect( iModeToggler, SIGNAL( toggleErrorSending( bool ) ), this, SLOT( toggleErrorSending( bool ) ) );
		iModeToggler->start();
	}
}

bool DeviceManager::setCurrentOperationMode()
{
	qDebug() << "DeviceManager::setCurrentOperationMode()";
	QStringList modeResult;
	if ( !iDataStorage->readData( iDataStorage->dataSectionToString( DeviceDataStorage::DeviceMode ), modeResult ) )
	{
		errorSender( ModeNotFetched );
		return false;
	}
	else
	{
		if ( !modeResult.empty() )
		{
			iMode = ( OperationMode )modeResult.at( 0 ).toInt();
			if ( iMode == StandAloneModeInProgress )
			{
				if ( !finalizeStandAloneMode() )
					iMode = EmptyMode;
				else
					iMode = StandAloneMode;
			}
		}
		else //this must be the first time, so no mode info saved yet
			iMode = StandAloneMode;
	}
	return true;
}

bool DeviceManager::finalizeStandAloneMode()
{
	qDebug() << "DeviceManager::finalizeStandAloneMode()";
	if ( !OperationModeToggler::storeOperationMode( StandAloneMode, iDataStorage ) || !iConfigurator->toggleInitScript( false ) ) {
		errorSender( DeviceManager::ModeNotStored );
		return false;
	}
	return true;
}

void DeviceManager::HWKeyFullScreenPressed()
{
	qDebug() << "DeviceManager::HWKeyFullScreenPressed()";
	handleHWKeyPresses( false );
	// no more key presses before this one is handled
	//handleKeyPresses( false );
	OperationMode nextMode;
	switch ( iMode ) {
		case KioskMode:
			nextMode = StandAloneMode;
			break;
		case StandAloneMode:
			nextMode = KioskMode;
			break;
		default:
			nextMode = EmptyMode;
			break;
	}
	if ( nextMode != EmptyMode )
		emit changeModeOrdered( nextMode );
}

void DeviceManager::errorSender( DeviceManager::ErrorCode aErrorCode, const QString &aAddInfo )
{
	qDebug() << "DeviceManager::errorSender( ErrorCode, QString & )";
	qDebug() << "DeviceManager::errorSender: aErrorCode == " << aErrorCode << " " << aAddInfo;
	if ( !iSendErrorMessages )
		return;

	emit error( ERROR_BASE + ( int )aErrorCode, aAddInfo ); 
}

void DeviceManager::toggleErrorSending( bool aToggle )
{
	qDebug() << "DeviceManager::toggleErrorSending( bool )";
	iSendErrorMessages = aToggle;
}

void DeviceManager::modeChanged()
{
	qDebug() << "DeviceManager::modeChanged()";
	if( iModeToggler != 0 ) {
		if ( iModeToggler->success() ) {//mode changing went well
			emit changingMode( "Restarting device" );
			sleep( 2 );
			iConfigurator->restartDevice();
		}
		else
			emit changeModeFailed();
		delete iModeToggler;
		iModeToggler = 0;
	}
	handleHWKeyPresses( true );
}

void DeviceManager::handleHWKeyPresses( bool aToggle )
{
	qDebug() << "DeviceManager::handleHWKeyPresses( bool )";
	if( aToggle )
		connect( iHWKeyListener, SIGNAL( HWKeyFullScreenPressed() ), this, SLOT( HWKeyFullScreenPressed() ) );
	else
		disconnect( iHWKeyListener, SIGNAL( HWKeyFullScreenPressed() ), this, SLOT( HWKeyFullScreenPressed() ) );
}
