#ifndef DEVICEMANAGER_H_
#define DEVICEMANAGER_H_

#include <QObject>
#include <QStringList>
#include <QMetaType>

class AlarmSender;
class HWKeyListener;
class StartupSettings;
class DeviceDataStorage;
class DeviceConfigurator;
class OperationModeToggler;

static const int ERROR_BASE=200;

//! DeviceControl class. The main class of the DeviceControl.
/*!
 * DeviceControl class. The main class of the DeviceControl. Responsible to communicate between the
 * BusinessLogic and the device. Takes care of the operation mode changes of the application.
 */
class DeviceManager : public QObject
{
	Q_OBJECT

public:
	//! Enumeration of device modes
	/*!
	 * Enumeration of device modes
	 */
	enum OperationMode
	{
		KioskMode, /*!< Device is in kiosk mode. */
		StandAloneModeInProgress, /*!< Device is in stand alone mode. */
		StandAloneMode, /*!< Device is in stand alone mode. */
		EmptyMode /*!< Application cannot read the mode. */
	};

	//! Enumeration of errors
	/*!
	 * Enumeration of errors
	 */
	
	enum ErrorCode
	{
		FileCreationFailed, /*!< File couldn't be created. */
		OldAlarmsNotRemoved, /*!< Previously sent old alarm events cannot be removed. */
		NewAlarmsNotSent, /*!< New alarms cannot be sent. */
		NewAlarmsNotStored, /*!< Information about new sent alarms cannot be stored. */
		ScreenSettingsNotStored, /*!< Configuration parameters of screen options cannot be stored. */
		ScreenSettingsNotFetched, /*!< Configuration parameters of screen options cannot be fetched. */
		ScreenSettingsNotChanged, /*!< Configuration parameters of screen options cannot be changed. */
		KeySettingsNotFetched, /*!< Configuration parameters of hw key options cannot be fetched. */
		KeySettingsNotStored, /*!< Configuration parameters of hw key options cannot be stored. */
		KeySettingsNotChanged, /*!< Configuration parameters of hw key options cannot be changed. */
		InitScriptNotChanged, /*!< Init script to auto launch the application was not registered/unregistered */
		ModeNotFetched, /*!< Information about the current operation mode cannot be fetched */
		ModeNotStored, /*!< Information about the new opration mode cannot be stored */
		DeviceNotRestarted /*!< Device cannot be restarted */
	};

public:
	//! Constructor.
	/*!
	 * Constructor to initialize a DeviceManager instance.
	 * \param aSettings Pointer to the start up configuration settings.
	 */
	DeviceManager( StartupSettings *aSettings );
	//! Destructor.
	virtual ~DeviceManager();
	//! Creates instances of AlarmSender, DeviceConfigurator, DeviceDataStorage and HWKeyListener classes.
	/*!
	 * Creates instances of AlarmSender, DeviceConfigurator, DeviceDataStorage and HWKeyListener classes.
	 * Connects their signals to the correct errorSender( ... ) slot.
	 */
	void initDeviceManager();
	//! Gets the current operation mode.
	/*!
	 * Gets the current operation mode of the application.
	 * \return enum value of the current operation mode.
	 */
	DeviceManager::OperationMode currentOperationMode();
	//! Gets the string value of an operation mode.
	/*!
	 * Gets the string value of an operation mode.
	 * \param aMode The enum value of the operation mode.
	 * \return QString value of the current operation mode.
	 */
	QString operationModeToString( OperationMode aMode );
	//! Changes the operation mode.
	/*!
	 * Changes the operation mode.
	 */
	void changeMode();
	void handleHWKeyPresses( bool aToggle );

signals:
	//! Signal. Emitted if user tries to change the operation mode.
	/*!
	 * Signal. Emitted if user tries to change the operation mode.
	 * \param aMode The operation mode that user wants to activate.
	 */
	void changeModeOrdered( DeviceManager::OperationMode aMode );
	//! Signal. Emitted if an error happens.
	/*!
	 * Signal. Emitted if an error happens.
	 * \param aCode An error code defined by DeviceManager.
	 * \param aAddInfo Possible additional information.
	 */
	void error( int aCode, const QString &aAddInfo );
	//! Signal. Emitted during the operation mode change to explain the current sub-change.
	/*!
	 * Signal. Emitted during the operation mode change to explain the current sub-change. 
	 * \param aMessage Explains the sub-change
	 */
	void changingMode( const QString &aMessage );
	//! Signal. Emitted if the operation mode change fails.
	/*!
	 * Signal. Emitted if the operation mode change fails.
	 */
	void changeModeFailed();

private slots:
	//! Slot. Handles "full screen"-hardware key presses.
	/*!
	 * Slot. Handles "full screen"-hardware key presses. Checks the current operation mode and concludes
	 * the next (desired) operation mode to be set. Emits a changeModeOrdered( DeviceManager::OperationMode )
	 * signal.
	 */
	void HWKeyFullScreenPressed();
	//! Slot. Sends errors.
	/*!
	 * Slot. Sends errors.
	 * \param aErrorCode The error code.
	 * \param aAddInfo The possible additional error text.
	 */
	void errorSender( DeviceManager::ErrorCode aErrorCode, const QString &aAddInfo = "" );
	//! Slot. Is called after the operation mode changing is finished.
	/*!
	 * Slot. Is called after the operation mode changing is finished. If everything went fine, restarts
	 * the device. 
	 */
	void modeChanged();
	//! Slot. Toggles an attribute that indicates whether the errors should be sent.
	/*!
	 * Slot. Toggles an attribute that indicates whether the errors should be sent.
	 * \param aToggle Indicates if the errors are wanted to be shown.
	 */
	void toggleErrorSending( bool aToggle );

private:
	//! Updates the internal indicator of the current operation mode.
	/*!
	 * Updates the internal indicator of the current operation mode by asking the DeviceDataStorage to
	 * read it from the internal data storage. Calls finalizeStandAloneMode() method in case the mode is
	 * StandAloneModeInProgress.
	 * \return True if operation mode fetching succeeds; otherwise, false.
	 */
	bool setCurrentOperationMode();
	//! Asks DeviceConfigurator to remove the deactivate script of the application.
	/*!
	 * Asks DeviceConfigurator to remove the deactivate script of the application. Also asks
	 * DeviceDataStorage to store the current operation mode (StandAloneMode) .
	 * \return True if operation mode storing and deactivation of the init script succeed; otherwise, false.
	 */
	bool finalizeStandAloneMode();

private:
	AlarmSender *iAlarmSender;
	HWKeyListener *iHWKeyListener;
	StartupSettings *iSettings;
	DeviceDataStorage *iDataStorage;
	DeviceConfigurator *iConfigurator;
	OperationModeToggler *iModeToggler;

	OperationMode iMode;
	bool iSendErrorMessages;

};

Q_DECLARE_METATYPE( DeviceManager::ErrorCode )

#endif /*DEVICEMANAGER_H_*/
