#include <QtTest/QtTest>
#include "Configuration.h"
#include "ConnectionSettings.h"
#include "DisplaySettings.h"
#include "StartupSettings.h"
#include "Room.h"
#include "TestConfiguration.h"

const QString sAdminPw = "admin";
const QString sUsername = "username";
const QString sServerPw = "password"; 
const QString sServerUrl = "jklexch01.ixonos.com";
const unsigned int sRefreshInterval = 30;
const DisplaySettings::DateFormat sDateFormat = DisplaySettings::LongDateFormat;
const DisplaySettings::TimeFormat sTimeFormat = DisplaySettings::TwelveHoursTimeFormat;
const DisplaySettings::DaysInSchedule sDaysInSchedule = DisplaySettings::WholeWeekInSchedule;
const QTime sDayStartsAt = QTime(6,0);
const QTime sDayEndsAt = QTime(18,0);
const int sScreensaver = 1;

const bool sIsPowersavingEnabled = false;
const QTime sTurnOnAt = QTime(5,45);
const QTime sTurnOffAt = QTime(18,15);



void TestConfiguration::initTestCase()
{
	iConf = Configuration::instance();
	QVERIFY( iConf != 0 );

	// Set values to empty configuration
	QList<Room*> testrooms;
	testrooms.append( new Room( "Taurus", "meetingroom.taurus_jyv@ixonos.com" ) );
	testrooms.append( new Room( "Pegasus", "meetingroom.pegasus_jyv@ixonos.com" ) );
	testrooms.append( new Room( "Hercules", "meetingroom.hercules@ixonos.com" ) );
	iConf->setRooms(testrooms);
	iConf->setAdminPassword(sAdminPw);

	// connection
	iConf->connectionSettings()->setServerUrl(sServerUrl);
	iConf->connectionSettings()->setUsername(sUsername);
	iConf->connectionSettings()->setPassword(sServerPw);
	iConf->connectionSettings()->setRefreshInterval(sRefreshInterval);
	
	// display
	iConf->displaySettings()->setDateFormat(sDateFormat);
	iConf->displaySettings()->setTimeFormat(sTimeFormat);
	iConf->displaySettings()->setDayStartsAt(sDayStartsAt);
	iConf->displaySettings()->setDayEndsAt(sDayEndsAt);
	iConf->displaySettings()->setDaysInSchedule(sDaysInSchedule);
	iConf->displaySettings()->setScreensaver(sScreensaver);
	
	// startup
	iConf->startupSettings()->setPowersavingEnabled(sIsPowersavingEnabled);
	iConf->startupSettings()->setTurnOnAt(sTurnOnAt);
	iConf->startupSettings()->setTurnOffAt(sTurnOffAt);
}

void TestConfiguration::cleanupTestCase()
{
	delete iConf;
	iConf = NULL;
}

void TestConfiguration::init()
{
}

void TestConfiguration::cleanup()
{
}

void TestConfiguration::testInstance()
{
	QCOMPARE( iConf, Configuration::instance() );
}

void TestConfiguration::testSave()
{
	// Just check there is no crash
	// The following testcases make sure the settings are saved correctly
	iConf->save();
/*	
	// delete old configuration 
	delete iConf;
	iConf = NULL;
	// ...and create new instance
	iConf = Configuration::instance();
	QVERIFY( iConf != 0 ); */
}


void TestConfiguration::testDefaultRoom()
{
	
	Room* room = iConf->defaultRoom();
	QVERIFY( room != 0 );
	QCOMPARE( room->name(), QString("Taurus") );
	QCOMPARE( room->address(), QString("meetingroom.taurus_jyv@ixonos.com") );
}

void TestConfiguration::testConnectionSettings()
{
	ConnectionSettings* conn = iConf->connectionSettings();
	QVERIFY( conn != 0 );
	QCOMPARE( conn->serverUrl().toString(), sServerUrl );
	QCOMPARE( conn->username(), sUsername );
	QCOMPARE( conn->password(), sServerPw );
	QCOMPARE( conn->refreshInterval(), sRefreshInterval );
}

void TestConfiguration::testLanguageCode()
{
	QCOMPARE( iConf->languageCode(), QString( "EN" ) );
}

void TestConfiguration::testRooms()
{
	QList<Room*> testrooms;
	testrooms.append( new Room( "Taurus", "meetingroom.taurus_jyv@ixonos.com" ) );
	testrooms.append( new Room( "Pegasus", "meetingroom.pegasus_jyv@ixonos.com" ) );
	testrooms.append( new Room( "Hercules", "meetingroom.hercules@ixonos.com" ) );
	
	QList<Room*> rooms = iConf->rooms();
	QVERIFY( rooms.count() == 3 );
	QCOMPARE( rooms[0], iConf->defaultRoom() );
	for ( int i = 0; i < rooms.count(); i++ )
	{
		Room* room = rooms[i];
		Room* testroom = testrooms[i];
		QCOMPARE( room->name(), testroom->name() );
		QCOMPARE( room->address(), testroom->address() );
	}
}

void TestConfiguration::testStartupSettings()
{
	StartupSettings *settings = iConf->startupSettings();

	QVERIFY( settings->isPowersavingEnabled() == sIsPowersavingEnabled );
	QCOMPARE( settings->turnOnAt(), sTurnOnAt );
	QCOMPARE( settings->turnOffAt(), sTurnOffAt );
}

void TestConfiguration::testDisplaySettings()
{
	DisplaySettings *settings = iConf->displaySettings();
	
	QVERIFY( settings->daysInSchedule() == sDaysInSchedule );
	QCOMPARE( settings->dayStartsAt(), sDayStartsAt );
	QCOMPARE( settings->dayEndsAt(), sDayEndsAt );
	QCOMPARE( settings->dateFormat(), QString( "dddd d MMMM yyyy" ) );
	QCOMPARE( settings->timeFormat(), QString( "hh:mm ap" ) );
}

void TestConfiguration::testAdminPassword()
{
	QCryptographicHash *hash = new QCryptographicHash( QCryptographicHash::Md5 );
	hash->addData( sAdminPw.toUtf8() );
	QByteArray password = hash->result();
	delete hash;

	QCOMPARE( iConf->adminPassword(), QString( password ) );
}
